
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
19481514
ActiveKids
Adapter
Alexey
Attrubute
AutoCommit
AutoInactiveDestroy
Avatica
COMMITED
CachedKids
ChildHandles
DBD
DSN
Default
Denis
ErrCount
FetchHashKeyName
HandleError
HandleSetErr
Ibaev
InactiveDestroy
Ivan
MaxRetries
NULLABLE
ParamValues
Phoenix
Postgres
PrintError
PrintWarn
Putintsev
RaiseError
ReadOnly
SQLSTATE
ShowErrorStatement
Stavrov
TaintIn
TaintOut
TraceLevel
TransactionIsolation
UNCOMMITED
UserAgent
avatica
dbh
dionys
errstr
lib
logioniz
nullable
protobuf
sequnce
uid
uid66
